/*
   AngelCode Windows Wrapper Library
   Copyright (c) 2004-2005 Andreas Jnsson
  
   This software is provided 'as-is', without any express or implied 
   warranty. In no event will the authors be held liable for any 
   damages arising from the use of this software.

   Permission is granted to anyone to use this software for any 
   purpose, including commercial applications, and to alter it and 
   redistribute it freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you 
      must not claim that you wrote the original software. If you use
	  this software in a product, an acknowledgment in the product 
	  documentation would be appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and 
      must not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source 
      distribution.
  
   Andreas Jnsson
   andreas@angelcode.com
*/



#include "ac_edit.h"
#include <assert.h>

acCEdit::acCEdit() : acCWindow()
{
}

int acCEdit::Create(DWORD style, RECT *rc, acCWindow *parent, UINT id)
{
	DWORD exStyle = WS_EX_CLIENTEDGE;
	HWND parentWnd = parent ? parent->GetHandle() : 0;
	HWND hEdit = CreateWindowEx(exStyle, "EDIT", "", style | WS_CHILD, rc->left, rc->top, rc->right - rc->left, rc->bottom - rc->top, parentWnd, 0, GetModuleHandle(0), 0);
	if( hEdit == 0 ) 
		return -1;

	if( Subclass(hEdit) < 0 )
		return -1;

	// TODO: Set control id

	return 0;
}

LRESULT acCEdit::MsgProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	return DefWndProc(msg, wParam, lParam);
}

int acCEdit::AppendText(const char *text)
{
	// Block updates to avoid flickering
	LockWindowUpdate(hWnd);

	// Store current position...
	int line = SendMessage(hWnd, EM_GETFIRSTVISIBLELINE, 0, 0);

	// and selection
	int start, end;
	SendMessage(hWnd, EM_GETSEL, (WPARAM)&start, (LPARAM)&end);

	// Move to the end of the text...
	int length = SendMessage(hWnd, WM_GETTEXTLENGTH, 0, 0); 

	SendMessage(hWnd, EM_SETSEL, length, length);

	// and add the new text
	SendMessage(hWnd, EM_REPLACESEL, FALSE, (LPARAM)text); 

	// Restore selection...
	SendMessage(hWnd, EM_SETSEL, start, end);

	// and position
	HWND hFocus = GetFocus();
	if( hFocus == hWnd )
	{
		// Scroll to the beginning of the text
		SendMessage(hWnd, EM_LINESCROLL, 0, 0x80000000);

		// Scroll to the same position as before
		SendMessage(hWnd, EM_LINESCROLL, 0, line);
	}

	// Allow updates
	LockWindowUpdate(0);

	return 0;
}

int acCEdit::GetLineCount()
{
	return SendMessage(hWnd, EM_GETLINECOUNT, 0, 0);
}

int acCEdit::DeleteLines(int first, int last)
{
	assert( last >= first );

	// Block updates to avoid flickering
	LockWindowUpdate(hWnd);

	// Store current position...
	int line = SendMessage(hWnd, EM_GETFIRSTVISIBLELINE, 0, 0);

	// and selection
	int start, end;
	SendMessage(hWnd, EM_GETSEL, (WPARAM)&start, (LPARAM)&end);
	
	// Get character positions for the lines we want to delete
	int c1 = SendMessage(hWnd, EM_LINEINDEX, first, 0);
	int c2 = SendMessage(hWnd, EM_LINEINDEX, last, 0);

	// Select this range...
	SendMessage(hWnd, EM_SETSEL, c1, c2);

	// and delete
	SendMessage(hWnd, EM_REPLACESEL, FALSE, (LPARAM)"");

	// Update stored positions
	if( line >= last ) line -= last - first;
	else if( line >= first ) line = first;

	if( start >= c2 ) start -= c2 - c1;
	else if( start >= c1 ) start = c1;

	if( end >= c2 ) end -= c2 - c1;
	else if( end >= c1 ) start = c1;

	// Restore selection...
	SendMessage(hWnd, EM_SETSEL, start, end);

	// and position
	if( hWnd == GetFocus() )
	{
		// Scroll to the beginning of the text
		SendMessage(hWnd, EM_LINESCROLL, 0, 0x80000000);

		// Scroll to the same position as before
		SendMessage(hWnd, EM_LINESCROLL, 0, line);
	}

	// Allow updates
	LockWindowUpdate(0);

	return 0;
}
