#include <new.h>      // _set_new_handler
#include <new>        // bad_alloc
#include <crtdbg.h>
#include "application.h"

int __cdecl newhandler(unsigned int wantedMemory)
{
	// If the function returns non-zero the new operator 
	// will try once more to allocate memory. If the function
	// returns 0 the new operator will fail and return 0.

	// This lets us free non-vital resources, such as textures
	// and soundbuffers, so that more memory can be allocated.
	// I suggest setting a flag that tells the program that it
	// encountered "out of memory". This will allow it to save
	// user data and then quit. If the flag is already set when
	// entering the newhandler there are no more memory to free
	// and we throw an exception causing an abrupt ending of
	// of the program.

	// We should not throw another type of exception than 
	// bad_alloc, because we don't know who else might be trying 
	// to catch out of memory exception. Throwing another type
	// will disable their attempt to catch the exception.
	throw std::bad_alloc();

	// Return 0 for failure
	return 0;
}

int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
{
	// Report memory leaks at the end of the application (in debug mode)
	_CrtSetDbgFlag(_CRTDBG_LEAK_CHECK_DF|_CRTDBG_ALLOC_MEM_DF);
	_CrtSetReportMode(_CRT_ASSERT,_CRTDBG_MODE_FILE);
	_CrtSetReportFile(_CRT_ASSERT,_CRTDBG_FILE_STDERR);

	// Install the new handler to throw an 
	// exception on out of memory
	_set_new_handler(newhandler);

	return app.Run();
}

